// gmlTcl_Entity.h --
//
//    Declares the gmlTcl_TEntity class. A superclass that is specialized by gmlTcl_TClass
//      and gmlTcl_TObject.
//
//  Copyright (c) 2007 LIG/IIHM
//
//  See the file "gml_LicenseTerms.txt" for information on usage and redistribution
//  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
//
//  Created on July 20, 2007 (FB).

#ifndef __GMLTCL_ENTITY__
#define __GMLTCL_ENTITY__

extern "C" {
 #include "tcl.h"
}


#include "gmlTcl_Obj_Internal.h"

typedef enum {

  gmlTcl_kEntityState_normal,
  gmlTcl_kEntityState_inInit,
  gmlTcl_kEntityState_inDispose,
  gmlTcl_kEntityState_cmdDeleted

} gmlTcl_TEntityState;


class gmlTcl_TEntity
{
  public:

    // Public object methods

    int                     Init                   (gmlTcl_TEntityType  type,
                                                    Tcl_Interp*         interp,
                                                    Tcl_Obj*            name);

    virtual void            Dispose                ();
    

    int                     UpdateStringNames      (char*               newName,
                                                    Tcl_Interp*         interp          = (Tcl_Interp*)NULL);

    virtual int             CommandHandler         (Tcl_Interp*         interp,
                                                    int                 objc,
                                                    Tcl_Obj*            objv[]) = 0;

    void                    CommandDeleted         ();

    void                    Rename                 (Tcl_Interp*         interp,
                                                    char*               oldName,
                                                    char*               newName,
                                                    int                 flags);


    // Accessors, Modifiers

    gmlTcl_TEntityType      Type                   () { return fType;             }
    char*                   TypeString             () { return fTypeString;       }
    Tcl_HashTable*          Table                  () { return fTable;            }
    char*                   StringName             () { return fStringName;       }
    char*                   ProtectedString        () { return fProtectedString;  }
    Tcl_Command             Command                () { return fCmd;              }


    
  protected:

    // Object attributes

    gmlTcl_TEntityState     fState;           // current state of the entity.
    gmlTcl_TEntityType      fType;            // the type of this entity: class or object.
    char*                   fTypeString;      // the string "class" or "object".
    Tcl_HashTable*          fTable;           // the hash table from string name to pointer.
    int                     fInTable;         // true if the entity has been added to the global string to pointer hash table.
    char*                   fStringName;      // string name of the entity.
    char*                   fProtectedString; // string name protected with {} if the name contains spaces.
    Tcl_Command             fCmd;             // the token for the object creation command (that has the name of the entity).
};


#endif
