// gmlTcl_Method.h --
//
//    Declares the gmlTcl_TMethod class. An object of this class represents a
//    Tcl-level method of a class.
//
//  Copyright (c) 2007 LIG/IIHM
//
//  See the file "gml_LicenseTerms.txt" for information on usage and redistribution
//  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
//
//  Created on June 24, 2007 (FB).

#ifndef __GMLTCL_METHOD__
#define __GMLTCL_METHOD__

extern "C" {
  #include "tcl.h"
  #include "tclInt.h"
  #include "tclCompile.h"
}

class gmlTcl_TClass;
class gmlTcl_TObject;

class gmlTcl_TMethod
{
  public:

    // Public object methods

    int                     Init                   (Tcl_Interp*     interp,
                                                    gmlTcl_TClass*  theClass,
                                                    int             objc,
                                                    Tcl_Obj*        objv[]);
    
    void                    Dispose                ();

    int                     Execute                (Tcl_Interp*       interp,
                                                    gmlTcl_TObject*   theObject,
                                                    Tcl_Obj*          objName,
                                                    Tcl_Obj*          className,
                                                    Tcl_Obj*          methodName,
                                                    int               objc,
                                                    Tcl_Obj*          objv[],
                                                    int               constructorCall     = 0);

    // Accessors, Modifiers

    char*                   StringName             () { return fStringName;       }
    char*                   ProtectedString        () { return fProtectedString;  }
    gmlTcl_TClass*          Class                  () { return fClass;            }
    Tcl_Obj*                Body                   () { return fBody; }
    Tcl_Obj*                ArgList                () { return fArgList; }

    Tcl_Obj*                Args ();


  protected:

    // Object attributes

    gmlTcl_TClass*          fClass;                 // the class of this method
    char*                   fStringName;            // string name of the method
    char*                   fProtectedString;       // string name protected with {} if the name contains spaces.
    char*                   fProcName;              // the proc name (i.e. "class method") of this method
    int                     fRegistered;            // true if this method has been registered to its class
    Proc*                   fProc;                  // the tcl proc structure associated with this method
    int                     fFlags;                 // 1 = method is contructor, 2 = method is destructor

    Tcl_Obj*                fBody;                  // memorize the body for "gmlObject info body"
    Tcl_Obj*                fArgList;               // memorize the argument list for "gmlObject info arglist"
    Tcl_Obj*                fArgs;                  // memorize the arguments for "gmlObject info args"
};


#endif
